/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry.client.mock.junit;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.junit.rules.ExternalResource;
import org.ow2.petals.registry.client.api.PetalsRegistryClientFactory;
import org.ow2.petals.registry.client.mock.PetalsRegistryClientFactoryMock;
import org.ow2.petals.topology.generated.Topology;

/**
 * The {@link PetalsRegistryClientApi} rule allows creation of a mock implementation of Petals Registry Client API that
 * is guaranteed to be deleted when the test method finishes (whether it passes or fails):
 * 
 * <pre>
 * public static class HasPetalsRegistryClientApi {
 *  &#064;Rule
 *  public PetalsRegistryClientApi petalsRegistryClientApi = new PetalsRegistryClientApi(...);
 * 
 *  &#064;Test
 *  public void test...() throws IOException {
 *      final PetalsRegistryClientFactory petalsRegistryClientFactory = petalsRegistryClientApi.getPetalsRegistryClientFactory();
 *      ...
 *  }
 * }
 * </pre>
 */
public class PetalsRegistryClientApi extends ExternalResource {

    private PetalsRegistryClientFactory prcf = null;

    /**
     * The map in which topologies will be registered
     */
    private final Map<String, Topology> topologyMap = new HashMap<String, Topology>();

    @Override
    protected void before() throws Throwable {
        this.initializePetalsRegistryClientApi();
    }

    @Override
    protected void after() {
        try {
            this.destroyPetalsRegistryClientApi();
        } catch (final Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Initialize the mock implementation of the Petals Registry Client API
     */
    private void initializePetalsRegistryClientApi() throws Exception {

        System.setProperty(PetalsRegistryClientFactory.PROPERTY_NAME, PetalsRegistryClientFactoryMock.class.getName());

        this.prcf = PetalsRegistryClientFactory.newInstance();
        if (this.prcf instanceof PetalsRegistryClientFactoryMock) {
            ((PetalsRegistryClientFactoryMock) this.prcf).setTopologyMap(this.topologyMap);
        } else {
            throw new Exception("An invalid implementation was selected as Petals Registry Client API Factory: "
                    + this.prcf.getClass().getName());
        }
    }

    /**
     * Free the mock implementation of the Petals Registry Client API
     */
    private void destroyPetalsRegistryClientApi() throws Exception {

        this.prcf = null;
        System.clearProperty(PetalsRegistryClientFactory.PROPERTY_NAME);
    }

    public PetalsRegistryClientFactory getPetalsRegistryClientFactory() {
        return this.prcf;
    }

    /**
     * <p>
     * Register a topology into the Petals Registry Client API mock.
     * </p>
     * <p>
     * Note: All topologies registered are automatically unregistered when the {@link PetalsRegistryClientApi} is freed,
     * for example at the end of a unit test.
     * </p>
     * 
     * @param topologyId
     *            The topology identifier
     * @param topology
     *            The topology to register
     */
    public void registerTopology(final String topologyId, final Topology topology) {
        this.topologyMap.put(topologyId, topology);
    }

    /**
     * @return The registered topologies, as unmodifiable map.
     */
    public Map<String, Topology> getRegisteredTopology() {
        return Collections.unmodifiableMap(this.topologyMap);
    }

    /**
     * <p>
     * Unregister a topology from the Petals Registry Client API Mock.
     * </p>
     * 
     * @param topologyId
     *            The topology identifier to unregister.
     * @return <code>true</code> if the topology was correctly unregistered, <code>false</code> if the topology was not
     *         registered previously.
     */
    public boolean unregisterTopology(final String topologyId) {
        return this.topologyMap.remove(topologyId) != null;
    }
}
